@@ -138,7 +138,7 @@ PackageNamespace <- R6::R6Class("PackageNamespace",
 
             # if the function exists in the workspace, write the code to a file
             temp_file <- file.path(tempdir(), paste0(symbol, ".R"))
-            readr::write_lines(code, temp_file)
+            stringi::stri_write_lines(code, temp_file)
             list(
                 uri = path_to_uri(temp_file),
                 range = range(
@@ -172,6 +172,7 @@ PackageNamespace <- R6::R6Class("PackageNamespace",
 WORKSPACE <- "_workspace_"
 
 #' A class for storing global environment information
+#' @keywords internal
 GlobalEnv <- R6::R6Class("GlobalEnv",
     public = list(
         documents = NULL,
